#define LOCKING_TESTS

using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Data.SQLite;
using MbUnit.Framework;
using MbUnit.Framework.Reflection;


namespace SubSonic.Tests
{
    [TestFixture]
    public class SQLiteDataProviderTests01
    {
        private const string CONNECTION_STRING = @"Data Source=C:\_svn\subsonic\SubSonic.Tests\DBScripts\TestDbpubs.db";
        private const string CONNECTION_NAME = "PubsSQLite";
        private const string PROVIDER_NAME = "PubsSQLite";
        private SQLiteDataProvider _provider;
        private readonly SQLiteConnection _conn = new SQLiteConnection(CONNECTION_STRING);
        
        [RowTest]
        [Row(CONNECTION_NAME, "testdbpubs")]
        public void CreateConnectionTest(string connectionName, string datasource)
        {
            _provider = new SQLiteDataProvider();
            
            _provider.ConnectionStringName = connectionName;

            Reflector _reflector = new Reflector(_provider);

            object result = _reflector.InvokeMethod("CreateConnection");

            DbConnection db = result as DbConnection;

            Assert.IsNotNull(db, "Provider is returning a null connection.");
            Assert.AreEqual(db.DataSource.ToLower(), datasource, "Provider returned wrong datasource.");

            //Test overload 
            result = _reflector.InvokeMethod("CreateConnection", CONNECTION_STRING);

            db = result as DbConnection;

            Assert.IsNotNull(db, "Provider is returning a null connection.(2)");
            Assert.AreEqual(db.DataSource.ToLower(), datasource, "Provider returned wrong datasource.(2)");

        }

        [Test]
        public void AddParamsTests()
        {
            _provider = new SQLiteDataProvider();
            _conn.Open();

            SQLiteCommand cmm = new SQLiteCommand("SELECT * FROM AUTHORS", _conn);

            QueryCommand qry = new QueryCommand("SELECT * FROM AUTHORS");
            qry.Parameters.Add("@LNAME", "Dull");

            Reflector _reflector = new Reflector(_provider);
            _reflector.InvokeMethod("AddParams", qry, cmm);

            Assert.AreEqual(cmm.Parameters.Count, 1, "Provider did not add the parameter to the command.");
            Assert.AreEqual(cmm.Parameters[0].ParameterName, "@LNAME", "Provider did not set the correct parameter name to the command.");
            Assert.AreEqual(cmm.Parameters[0].Value, "Dull", "Provider did not set the correct parameter value to the command.");

            cmm.ExecuteNonQuery();

        }

        [RowTest]
        [Row("pubs")]
        [Row("name with spaces")]
        public void DelimitDbNameTest(string name)
        {
            _provider = new SQLiteDataProvider();

            Assert.AreEqual(_provider.DelimitDbName(name), string.Format("`{0}`", name), "Delimiting is wrong.");
        }

        [Test]
        public void DelimitDbNameTest2()
        {
            _provider = new SQLiteDataProvider();

            Assert.AreEqual(_provider.DelimitDbName(string.Empty), string.Empty, "Delimiting is wrong.");
            Assert.AreEqual(_provider.DelimitDbName(null), string.Empty, "Delimiting is wrong.");
        }

        [RowTest]
        [Row("SELECT * FROM AUTHORS WHERE AU_LNAME = ?", "?LNAME", "Dull")]
        public void GetReaderTests(string sql, string paramName, string paramValue)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand(sql);
            qry.Parameters.Add(paramName, paramValue);

            IDataReader rdr = _provider.GetReader(qry);

            Assert.IsNotNull(rdr, "GetReader returns a null reader.");
            Assert.IsTrue(rdr.Read(), "GetReader did not return any record.");
            Assert.AreNotEqual(rdr.FieldCount, 0, "GetReader returned zero fields.");
            Assert.AreEqual(rdr.FieldCount, 9, "GetReader returned an invalid number of fields.");
        }

        [RowTest]
        [Row("SELECT * FROM AUTHORS WHERE AU_LNAME = ? AND AU_FNAME = ?", "@LNAME", "Ringer", "@FNAME", "Albert")]
        public void GetDataSetTests(string sql, string paramName, string paramValue, string paramName2, string paramValue2)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand(sql);
            qry.Parameters.Add(paramName, paramValue);
            qry.Parameters.Add(paramName2, paramValue2);

            DataSet ds = _provider.GetDataSet(qry);

            Assert.IsNotNull(ds, "GetDataSet returns a null dataset.");
            Assert.AreNotEqual(ds.Tables.Count, 0, "GetDataset returned zero tables.");
            Assert.AreNotEqual(ds.Tables[0].Rows.Count, 0, "GetDataSet did not return any record.");
            Assert.AreEqual(ds.Tables[0].Rows.Count, 1, "GetDataSet did not return the correct number of records.");
            Assert.AreEqual(ds.Tables.Count, 1, "GetDataSet returned an invalid number of tables.");
        }

        [RowTest]
        [Row("SELECT COUNT(*) FROM AUTHORS WHERE AU_LNAME = ?", "@LNAME", "Dull")]
        public void ExecuteScalarTests(string sql, string paramName, string paramValue)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand(sql);
            qry.Parameters.Add(paramName, paramValue);

            object result = _provider.ExecuteScalar(qry);

            Assert.IsNotNull(result, "ExecuteScalar returns a null value.");
            Assert.AreNotEqual(Convert.ToInt32(result), 0, "ExecuteScalar did not return any record.");
            Assert.AreEqual(Convert.ToInt32(result), 1, "ExecuteScalar returned an invalid number of records.");
        }

        
        [RowTest]
        [Row("AUTHORS", "Author","au_id", 9, 1,TableType.Table)]
        [Row("EMPLOYEE", "Employee", "emp_id", 8,1, TableType.Table)]
        [Row("TITLEVIEW", "Titleview", "", 6, 0,TableType.View)]
        public void GetTableSchemaTests(string tableName, string displayName, string primaryKey, int columnCount, int PKCount, TableType tableType)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            TableSchema.Table result = _provider.GetTableSchema(tableName, tableType);

            Assert.IsNotNull(result, "GetTableSchemaTests returns a null value.");
            Assert.AreEqual(result.Columns.Count, columnCount, "GetTableSchemaTests returned an invalid number of column value.");
            Assert.AreEqual(result.PrimaryKeys.Length, PKCount, "GetTableSchemaTests returned an invalid number of primary keys value.");
            if (PKCount == 1)
                Assert.AreEqual(result.PrimaryKey.ColumnName, primaryKey, "GetTableSchemaTests returned an invalid value for DisplayName.");
            Assert.AreEqual(result.DisplayName, displayName, "GetTableSchemaTests returned an invalid value for DisplayName.");
        }

        [RowTest]
        [Row("INTEGER", DbType.Int64)]
        [Row("REAL", DbType.Single)]
        [Row("TEXT", DbType.String)]
        [Row("BLOB", DbType.Binary)]
        public void GetDbTypeTests(string typeName, DbType expected)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            DbType result = _provider.GetDbType(typeName);

            Assert.AreEqual(result, expected, "Wrong type was returned");
        }

        [RowTest]
        [Row("INTEGER", typeof(Int64))]
        [Row("REAL", typeof(Single))]
        [Row("TEXT", typeof(string))]
        [Row("BLOB", typeof(byte[]))]
        public void GetTypeTests(string typeName, Type expected)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Type result = _provider.GetType(typeName);

            Assert.AreEqual(result, expected, "Wrong type was returned");
        }

        [Test]
        public void GetTableNameListTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string[] result = _provider.GetTableNameList();

            Assert.AreEqual(result.Length, 12, "Wrong number of tables was returned");
        }

        [RowTest]
        [Row("JOBS", "employee", "job_id")]
        [Row("AUTHORS", "titleauthor", "au_id")]
        public void GetPrimaryKeyTableNamesTests(string tableName, string referredTable, string pkName)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            ArrayList result = _provider.GetPrimaryKeyTableNames(tableName);

            Assert.IsNotNull(result, "The returned list is null.");

            string[] values = (string[]) result[0];

            Assert.AreEqual(values[0], referredTable, "GetPrimaryKeyTableNames returned the wrong referring table.");
            Assert.AreEqual(values[1], pkName, "GetPrimaryKeyTableNames returned the wrong PK.");
        }

        [Test]
        public void GetPrimaryKeyTableNamesTestsMulti()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            ArrayList result = _provider.GetPrimaryKeyTableNames("publishers");

            Assert.IsNotNull(result, "The returned list is null.");

            string[,] expected = new string[3,2]
                {
                    {"pub_info", "pub_id"},
                    {"titles", "pub_id"},
                    {"employee", "pub_id"}
                };

            for(int n=0; n<3; n++)
            {
                string[] values = (string[])result[n];

                Assert.AreEqual(values[0], expected[n,0], "GetPrimaryKeyTableNames returned the wrong referring table.");
                Assert.AreEqual(values[1], expected[n,1], "GetPrimaryKeyTableNames returned the wrong PK.");
            }
        }

        [RowTest]
        [Row("JOBS", "employee", "job_id")]
        [Row("AUTHORS", "titleauthor", "au_id")]
        public void GetPrimaryKeyTablesTests(string tableName, string referredTable, string pkName)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            TableSchema.Table[] result = _provider.GetPrimaryKeyTables(tableName);

            Assert.IsNotNull(result, "The returned list is null.");

            Assert.AreEqual(result[0].Name, referredTable, "GetPrimaryKeyTables returned the wrong referring table.");
            Assert.AreEqual(result[0].ForeignKeys[0].ColumnName, pkName, "GetPrimaryKeyTables returned the wrong PK.");
        }

        [Test]
        public void GetPrimaryKeyTablesTestsMulti()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            TableSchema.Table[] result = _provider.GetPrimaryKeyTables("publishers");

            Assert.IsNotNull(result, "The returned list is null.");

            string[,] expected = new string[3, 2]
                {
                    {"pub_info", "pub_id"},
                    {"titles", "pub_id"},
                    {"employee", "job_id"}
                };

            for (int n = 0; n < 3; n++)
            {
                Assert.AreEqual(result[n].ForeignKeys[0].TableName, expected[n, 0], "GetPrimaryKeyTableNames returned the wrong referring table.");
                Assert.AreEqual(result[n].ForeignKeys[0].ColumnName, expected[n, 1], "GetPrimaryKeyTableNames returned the wrong PK.");
            }
        }

        [RowTest]
        [Row("au_id", "titleauthor", "authors")]
        public void GetForeignKeyTableNameTests(string fkColumnName, string tableName, string expectedTable)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string result = _provider.GetForeignKeyTableName(fkColumnName, tableName);

            Assert.IsNotNull(result, "The returned string is null.");

            Assert.AreNotEqual(result, string.Empty, "GetForeignKeyTableName returned an empty string.");

            Assert.AreEqual(result, expectedTable, "GetForeignKeyTableName returned the wrong referring table.");
        }

        [RowTest]
        [Row("au_id", "authors")]
        public void GetForeignKeyTableNameTests1(string tableName, string expectedTable)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string result = _provider.GetForeignKeyTableName(tableName);

            Assert.IsNotNull(result, "The returned string is null.");

            Assert.AreNotEqual(result, string.Empty, "GetForeignKeyTableName returned an empty string.");

            Assert.AreEqual(result, expectedTable, "GetForeignKeyTableName returned the wrong referring table.");
        }

        [Test, ExpectedException(typeof (Exception))]
        public void ExecuteTransactionTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            _provider.ExecuteTransaction(new QueryCommandCollection());
        }
        
        [Test, ExpectedException(typeof(NullReferenceException))]
        public void GetSelectSqlTests1()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            _provider.GetSelectSql(null);
        }

        [RowTest]
        [Row("AUTHORS", "SELECT  *  FROM main.`AUTHORS`;")]
        [Row("PUBLISHERS", "SELECT  *  FROM main.`PUBLISHERS`;")]
        [Row("TITLEVIEW", "SELECT  *  FROM main.`TITLEVIEW`;")]
        public void GetSelectSqlTestsSimple(string tableName, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);

            Assert.AreEqual(_provider.GetSelectSql(qry), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("AUTHORS", "SELECT DISTINCT  *  FROM main.`AUTHORS`;")]
        [Row("PUBLISHERS", "SELECT DISTINCT  *  FROM main.`PUBLISHERS`;")]
        [Row("TITLEVIEW", "SELECT DISTINCT  *  FROM main.`TITLEVIEW`;")]
        public void GetSelectSqlTestsDistinct(string tableName, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.IsDistinct = true;

            Assert.AreEqual(_provider.GetSelectSql(qry), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("SALES", "SELECT  *  FROM main.`SALES` WHERE `SALES`.`qty` >= ? AND `SALES`.`qty` <= ?;")]
        public void GetSelectSqlTestsBetweenValues(string tableName, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.AddBetweenValues("qty", 10, 20);

            Assert.AreEqual(_provider.GetSelectSql(qry), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("SALES", "SELECT  *  FROM main.`SALES` WHERE `SALES`.`ord_date` BETWEEN  ?  AND  ? ;")]
        public void GetSelectSqlTestsAddBetweenAnd(string tableName, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.AddBetweenAnd("ord_date", new DateTime(1994, 9, 1), new DateTime(1994, 09, 14));

            Assert.AreEqual(_provider.GetSelectSql(qry), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("AUTHORS", "contract", "0", "SELECT  *  FROM main.`AUTHORS` WHERE `AUTHORS`.`contract` = ?;")]
        public void GetSelectSqlTestsAddUpdateSetting(string tableName, string columnName, object value, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.AddWhere(columnName, Comparison.Equals, value);

            Assert.AreEqual(_provider.GetSelectSql(qry), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("AUTHORS", "contract", "0", " WHERE `AUTHORS`.`contract` = ?")]
        public void BuildWhereSQLiteTests(string tableName, string columnName, object value, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Reflector _reflector = new Reflector(_provider);

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.AddWhere(columnName, Comparison.Equals, value);

            object result = _reflector.InvokeMethod("BuildWhereSQLite", qry);

            Assert.AreEqual(result.ToString(), expectedSQL,
                            "The sql returned by GetSelectSql is wrong.");
        }

        [RowTest]
        [Row("DISCOUNTS", "INSERT INTO `DISCOUNTS`(discounttype,stor_id,lowqty,highqty,discount) VALUES(?,?,?,?,?) SELECT LAST_INSERT_ROWID() as newID")]
        public void GetInsertSqlTests(string tableName, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);

            Assert.AreEqual(_provider.GetInsertSql(qry), expectedSQL,
                            "The sql returned by GetInsertSql is wrong.");
        }

        [RowTest]
        [Row("SALES", "stor_id", "DELETE FROM `main`.`SALES` WHERE `SALES`.`stor_id` < ?")]
        public void GetDeleteSqliteTests(string tableName, string columnName, string expectedSQL)
        {
            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.WHERE(columnName, Comparison.LessThan, 6380);

            Assert.AreEqual(SQLiteDataProvider.GetDeleteSqlite(qry), expectedSQL,
                            "The sql returned by GetDeleteSqlite is wrong.");
        }

        //This is test is not finished but the idea was to test all the Comparison SQLs in one go.
        [Factory(typeof(string[]))]
        public IEnumerable GetDeleteSqliteTestsRows()
        {
            string[] row1 = { "d1" };
            string[] row2 = { "d2" };
            string[] row3 = { "d3" };

            string[][] arrays = {row1, row2, row3};
            return arrays;
        }

        [CombinatorialTest]
        public void GetDeleteSqliteTests2( [UsingEnum(typeof(Comparison))] Comparison comparisons
            /*[UsingFactories("GetDeleteSqliteTestsRows")] string[] expectedSQL*/)
        {
            Query qry = new Query("SALES", PROVIDER_NAME);
            qry.WHERE("stor_id", comparisons, 6380);

            Assert.AreNotEqual(SQLiteDataProvider.GetDeleteSqlite(qry), "",
                               "The sql returned by GetDeleteSqlite is empty.");
        }

        /// <summary>
        /// The SQL generation for the BuildCommand appears to be work
        /// in progress so I will leave it as it is (not the non escaped name in the update SQL).
        /// </summary>
        /// <param name="tableName"></param>
        /// <param name="type"></param>
        /// <param name="expectedSQL"></param>
        [RowTest]
        [Row("AUTHORS", QueryType.Select, "SELECT  *  FROM main.`AUTHORS`;")]
        [Row("AUTHORS", QueryType.Delete, "DELETE FROM `main`.`AUTHORS`")]
        [Row("AUTHORS", QueryType.Update, "UPDATE AUTHORS")]
        public void BuildCommandTests(string tableName, QueryType type, string expectedSQL)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            Query qry = new Query(tableName, PROVIDER_NAME);
            qry.QueryType = type;

            QueryCommand result = _provider.BuildCommand(qry);

            Assert.AreEqual(result.CommandSql, expectedSQL,
                            "The sql returned by BuildCommandTests is wrong.");
        }

        [Test]
        public void GetDbCommandTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand("SELECT * FROM AUTHORS WHERE au_id = ?");
            qry.AddParameter("@au_id", "274-80-9391", DbType.String, ParameterDirection.Input);

            DbCommand result = _provider.GetDbCommand(qry);

            Assert.AreEqual(result.Parameters.Count, 1,
                            "GetDbCommand returned a wrong number of parameters.");

            IDataReader rdr = result.ExecuteReader(CommandBehavior.CloseConnection);

            Assert.IsTrue(rdr.Read(), "GetDbCommand did not return any records.");
            Assert.IsFalse(rdr.Read(), "GetDbCommand returned too many records.");
        }

        [RowTest]
        [Row("SELECT * FROM AUTHORS WHERE AU_LNAME = ?", "?LNAME", "Dull")]
        [Row("DELETE FROM AUTHORS WHERE AU_LNAME = ?", "?LNAME", "Green")]
        [Row("UPDATE AUTHORS SET au_lname = '' WHERE AU_LNAME = ?", "?LNAME", "Green")]
        public void GetCommandTests(string sql, string paramName, string paramValue)
        {
            _provider = new SQLiteDataProvider();

            QueryCommand qry = new QueryCommand(sql);
            qry.Parameters.Add(paramName, paramValue);

            SQLiteCommand cmm = (SQLiteCommand)_provider.GetCommand(qry);

            Assert.IsNotNull(cmm);
            Assert.AreNotEqual(cmm.Parameters.Count, 0, "Provider did not add the parameter to the command.");
            Assert.AreEqual(cmm.Parameters[0].ParameterName, paramName, "Provider did not set the correct parameter name to the command.");
            Assert.AreEqual(cmm.Parameters[0].Value, paramValue, "Provider did not set the correct parameter value to the command.");
        }

#if LOCKING_TESTS 

        /******************************************************************************************************************
         * These tests may lock the database file
        /******************************************************************************************************************/

        
        [RowTest, RollBack]
        [Row("DELETE FROM AUTHORS WHERE AU_LNAME=?", "@LNAME", "Green")]
        public void ExecuteQueryTests(string sql, string paramName, string paramValue)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand(sql);
            qry.Parameters.Add(paramName, paramValue);

            object result = _provider.ExecuteQuery(qry);

            Assert.IsNotNull(result, "ExecuteQuery returns a null value.");
            Assert.AreNotEqual(Convert.ToInt32(result), 0, "ExecuteQuery did not return any record.");
            Assert.AreEqual(Convert.ToInt32(result), 1, "ExecuteQuery returned an invalid number of records.");
        }

        [Test, ExpectedException(typeof(Exception))]
        public void ExecuteTransactionTests1()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            _provider.ExecuteTransaction(null);
        }

        [Test, ExpectedException(typeof(SQLiteException))]
        public void ExecuteTransactionTests2()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommandCollection cmds = new QueryCommandCollection();
            cmds.Add(new QueryCommand("SELECT * FROM NONEXISTINGTABLE"));

            _provider.ExecuteTransaction(cmds);
        }

        [Test]
        public void ExecuteTransactionTests3()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommandCollection cmds = new QueryCommandCollection();
            cmds.Add(new QueryCommand("SELECT * FROM AUTHORS"));
            cmds.Add(new QueryCommand("SELECT * FROM PUBLISHERS"));

            _provider.ExecuteTransaction(cmds);
        }
#endif //LOCKING_TESTS

        [Test]
        public void GetDataSetTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            QueryCommand qry = new QueryCommand("SELECT * FROM AUTHORS WHERE au_id = ?");
            qry.AddParameter("@au_id", "274-80-9391", DbType.String, ParameterDirection.Input);

            DataSet ds = _provider.GetDataSet(qry);
            
            Assert.IsNotNull(ds, "GetDbCommand did not return any records.");
        }

        [RowTest]
        [Row("employee", 2, "publishers", "jobs", "")]
        [Row("sales", 2, "titles", "stores", "")]
        [Row("titleauthor", 2, "titles", "authors", "")]
        public void GetForeignKeyTablesTests(string tableName, int number, string name1, string name2, string name3)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string[] names = _provider.GetForeignKeyTables(tableName);

            Assert.AreEqual(names.Length, number, "Wrong number of tables returned.");

            if(number >= 1)
                Assert.AreEqual(names[0], name1, "Invalid value returned.");

            if (number >= 2)
                Assert.AreEqual(names[1], name2, "Invalid value returned.");

            if (number == 3)
                Assert.AreEqual(names[2], name3, "Invalid value returned.");
        }

        [RowTest]
        [Row("emp_id", "employee")]
        [Row("au_id", "authors")]
        [Row("####", "")]
        public void GetTableNameByPrimaryKeyTests(string pkName, string tableName)
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string names = _provider.GetTableNameByPrimaryKey(pkName, "");

            Assert.AreEqual(names, tableName, "Wrong table returned.");

        }

        [Test]
        public void GetViewNameListTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string[] views = _provider.GetViewNameList();

            Assert.AreEqual(views.Length, 1, "Returned the wrong number of views.");
            Assert.AreEqual(views[0], "titleview", "Returned the wrong view name.");
        }

        [Test]
        public void ScriptDataTests()
        {
            _provider = new SQLiteDataProvider();
            _provider.ConnectionStringName = CONNECTION_NAME;

            string data = _provider.ScriptData("employee");

            Assert.AreNotEqual(data.Length, 0, "Returned an empty string.");
            
        }
    }
}